const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Slider = imports.ui.slider;
const Lang = imports.lang;
const ExtensionUtils = imports.misc.extensionUtils;
const Gio = imports.gi.Gio;
const Clutter = imports.gi.Clutter;
const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const Duolingo = Me.imports.duolingo.Duolingo;
const Reminder = Me.imports.reminder.Reminder;

const Util = imports.misc.util;
const FLAGS = Me.imports.flagsKeys.flags;
const Utils = Me.imports.utils;
const Settings = Convenience.getSettings();
const Constants = Me.imports.constants;

const Gettext = imports.gettext;
const _ = Gettext.gettext;

let icon_size = 16;
let menu_width = 250;

const DuolingoMenuButton = new Lang.Class({
    Name: 'Duolingo.DuolingoMenuButton',
    Extends: PanelMenu.Button,

	_init: function() {
        this.parent(0.0, 'duolingo');

        Gettext.textdomain(Me.uuid);
	    Gettext.bindtextdomain(Me.uuid, Me.dir.get_child('locale').get_path());

        this.reminder = null;
		this.duolingo = new Duolingo(Settings.get_string(Constants.SETTING_USERNAME));
		this.duolingo.get_raw_data(Lang.bind(this, this._create_menus));
	},

	_create_menus: function(error) {
		if (Settings.get_boolean(Constants.SETTING_HIDE_WHEN_DAILY_GOAL_REACHED) && this.duolingo.is_daily_goal_reached()) {
			this.destroy();
			return;
		}

		if(error) {
			Main.notify(Constants.LABEL_NOTIFICATION_TITLE, error);
			this._init_icon(Constants.ICON_DUOLINGO_ALERT);
			this._init_duolingo_menu();
		} else {
    		this._init_icon(Constants.ICON_DUOLINGO);
    		this._init_duolingo_menu();

    		/* display profile menu */
    		this.todays_improvement = new St.Label({
                y_align: Clutter.ActorAlign.CENTER});
    		this.profile_menu = new PopupMenu.PopupBaseMenuItem();

    		this.profile_menu.actor.add(this.todays_improvement);

    		let streak_label = new St.Label({
                x_align: Clutter.ActorAlign.CENTER,
                y_align: Clutter.ActorAlign.CENTER });
    		if (this.duolingo.get_streak() != 0) {
    			streak_label.text =  this.duolingo.get_streak().toString();
                if (this.duolingo.is_frozen()) {
                    streak_label.style_class = Constants.STYLE_STREAK_FROZEN;
                } else {
                    streak_label.style_class = Constants.STYLE_STREAK_NOT_FROZEN;
                }

    		} else {
    			streak_label.text = '';
    		}
    		this.profile_menu.actor.add(streak_label, {expand: true});

    		this.menu.addMenuItem(this.profile_menu);
    		this._set_todays_improvement();
    		this._display_lingots();
            this._display_double_or_nothing();

    		/* display language menus */
    		this._add_language_menus();

            // initiate reminder
            this._initiate_reminder();
        }
        this._finalize_menu_icon();
	},

	_init_icon: function(path) {
		this.hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
		let gicon = Gio.icon_new_for_string(path);
		let icon = new St.Icon({gicon: gicon, icon_size: icon_size});
        this.hbox.add_child(icon);
        this.actor.add_style_class_name('panel-status-button');
		this.actor.add_child(this.hbox);
	},

	_init_duolingo_menu: function() {
		/* Duolingo menu */
		let link_menu = new PopupMenu.PopupBaseMenuItem();
		link_menu.actor.width = menu_width;
		let link_label = new St.Label({ text: 'Duolingo.com', x_align: Clutter.ActorAlign.CENTER });
        link_label.add_style_class_name(Constants.STYLE_DUOLINGO_LINK);
		link_menu.actor.add(link_label, { expand: true });
		link_menu.connect('activate', function() {
			Util.spawn([Settings.get_string(Constants.SETTING_OPENING_BROWSER_COMMAND), Constants.URL_DUOLINGO_HOME]);
		});

		/* refresh button */
		let refresh_icon = new St.Icon({
			icon_name: 'view-refresh-symbolic',
			style_class: 'system-actions-icon',
			icon_size: icon_size
		});
		this.refresh_button = new St.Button({child: refresh_icon});
		link_menu.actor.add(this.refresh_button, {expand: false});

		/* Preferences button */
		let preferences_icon = new St.Icon({
			icon_name: 'system-run-symbolic',
			style_class: 'system-actions-icon',
			icon_size: icon_size
		});
		let preferences_button = new St.Button({child: preferences_icon});
		preferences_button.connect('clicked', Lang.bind(this, function() {
            this.emit(Constants.EVENT_PREFERENCES);
		}));
		link_menu.actor.add(preferences_button, {expand: false});

		this.menu.addMenuItem(link_menu);
	},

    get_refresh_button: function() {
        return this.refresh_button;
    },

    _initiate_reminder: function() {
        this.reminder = new Reminder(this.duolingo);
        if (!this.duolingo.is_daily_goal_reached()) {
            this.reminder.start();
        }
    },

    _stop_reminder: function() {
        if (this.reminder != null)
            this.reminder.stop();
    },

	_add_language_menus: function() {
        let languages = this.duolingo.get_languages();
		for (let l in languages) {
			let m = new LanguageSubMenu(languages[l], this.duolingo);
			this.menu.addMenuItem(m);
		}
	},

	_display_lingots: function() {
        if(this.duolingo.get_double_or_nothing_status() == null || Settings.get_boolean(Constants.SETTING_SHOW_LINGOTS)) {
            let lingots = this.duolingo.get_lingots();
        	let gicon = Gio.icon_new_for_string(Constants.ICON_LINGOTS);
        	let lingots_icon = new St.Icon({
                gicon: gicon,
                icon_size: icon_size,
                y_align: Clutter.ActorAlign.CENTER});
        	let lingots_label = new St.Label({
                y_align: Clutter.ActorAlign.CENTER,
                text: Utils.formatThousandNumber(lingots.toString())});
            lingots_label.add_style_class_name(Constants.STYLE_LINGOTS_LABEL);
        	this.profile_menu.connect('activate', this._open_lingots_link);
            this.profile_menu.actor.add(lingots_icon);
        	this.profile_menu.actor.add(lingots_label);
        }
	},

    _display_double_or_nothing: function() {
        let double_or_nothing = this.duolingo.get_double_or_nothing_status();
        if (double_or_nothing != null) {
            let gicon = Gio.icon_new_for_string(Constants.ICON_FIRE);
        	let fire_icon = new St.Icon({
                gicon: gicon,
                icon_size: icon_size,
                y_align: Clutter.ActorAlign.CENTER});
    		let double_or_nothing_label = new St.Label({
                y_align:Clutter.ActorAlign.CENTER,
                text: double_or_nothing + Constants.LABEL_XP_SEPARATOR + '7'});
            double_or_nothing_label.add_style_class_name(Constants.STYLE_DOUBLE_OR_NOTHING_LABEL);
        	this.profile_menu.connect('activate', this._open_lingots_link);
    		this.profile_menu.actor.add(fire_icon);
    		this.profile_menu.actor.add(double_or_nothing_label);
        }
	},

	_open_lingots_link: function() {
        Util.spawn([Settings.get_string(Constants.SETTING_OPENING_BROWSER_COMMAND), Constants.URL_DUOLINGO_STORE]);
	},

	_set_todays_improvement: function() {
		let improvement = this.duolingo.get_improvement();
		let daily_goal = this.duolingo.get_daily_goal();
		this.todays_improvement.text = improvement + Constants.LABEL_XP_SEPARATOR + daily_goal + ' XP';

		if (!this.duolingo.is_daily_goal_reached()) {
			this.hbox.get_child_at_index(0).style = 'color: ' + Settings.get_string(Constants.SETTING_ICON_COLOR_WHEN_DAILY_GOAL_NOT_REACHED) +';'
		} else {
			if(Settings.get_boolean(Constants.SETTING_CHANGE_ICON_COLOR_WHEN_DAILY_GOAL_REACHED)) {
				this.hbox.get_child_at_index(0).style = 'color: ' + Settings.get_string(Constants.SETTING_ICON_COLOR_WHEN_DAILY_GOAL_REACHED) +';'
			}
		}
	},

    _finalize_menu_icon: function() {
        // for the top right corner, use Main.panel.addToStatusArea() with an index as a 3rd parameter.
        // let index = 0; // 0, 1: normal in the queue, 2: just at the left of the main menu, -1: completely in the top right corner
        // let position = 'right';
        let index = parseInt(Settings.get_string(Constants.SETTING_ICON_INDEX));
        let position = Settings.get_string(Constants.SETTING_ICON_POSITION);
        Main.panel.addToStatusArea('duolingo', this, index, position);
        this.emit(Constants.EVENT_READY);
    },

    destroy: function() {
        this._stop_reminder();
		this.parent();
    },
});

const LanguageSubMenu = new Lang.Class({
    Name: 'Duolingo.LanguageMenu',
    Extends: PopupMenu.PopupSubMenuMenuItem,

	_init: function(language, duolingo) {
		this.parent(language[Constants.LANGUAGE_LABEL], true);
        this.language_code = language[Constants.LANGUAGE_CODE];

		/* display the flag */
		let flag_name = FLAGS[language[Constants.LANGUAGE_LABEL]];
		this.icon.gicon = Gio.icon_new_for_string(Constants.ICON_FLAG_PATH + flag_name);
		this.icon.icon_size = icon_size;

        // TODO: display star or change label color
		if (language[Constants.LANGUAGE_CURRENT_LANGUAGE]) {
			if (duolingo.get_count_learned_chapters() == duolingo.get_count_available_chapters()) {
				// let completed_icon = new St.Icon({
				// 	icon_name: 'starred-symbolic',
				// 	style_class: 'system-actions-icon',
				// 	icon_size: icon_size,
				// 	y_align: Clutter.ActorAlign.CENTER
				// });
				// this.actor.insert_child_at_index(completed_icon,3);
			}
		}

		/* Insert the current level of the language. 5 is the index of the last position in the sub menu */
		this.actor.insert_child_at_index(new St.Label({
            text: _('lvl. ') + language[Constants.LANGUAGE_LEVEL].toString(),
            y_align: Clutter.ActorAlign.CENTER
        }), 5);
		// this.refresh_button = new St.Button({child: refresh_icon});

		/* Add the menu displaying the global points of the language */
		let menu_total_points = new PopupMenu.PopupBaseMenuItem();
		menu_total_points.actor.add(new St.Label({text: _('Total'), x_expand: true, style: 'font-weight: bold;'}));
		menu_total_points.actor.add(new St.Label({text: Utils.formatThousandNumber(language[Constants.LANGUAGE_POINTS].toString()) + ' XP', style: 'font-weight: bold;'}));
		this.menu.addMenuItem(menu_total_points);

		let menu_next_level = new PopupMenu.PopupBaseMenuItem();
		menu_next_level.actor.add(new St.Label({text: _('Next level in'), x_expand: true}));
		menu_next_level.actor.add(new St.Label({text: Utils.formatThousandNumber(language[Constants.LANGUAGE_TO_NEXT_LEVEL].toString()) + ' XP'}));
		this.menu.addMenuItem(menu_next_level);

        if (language[Constants.LANGUAGE_CURRENT_LANGUAGE]) {
            let completion = new PopupMenu.PopupBaseMenuItem();
    		completion.actor.add(new St.Label({
				text: _('Completion'), 
				x_expand: true
			}));
            let label = duolingo.get_count_learned_chapters() + Constants.LABEL_XP_SEPARATOR + duolingo.get_count_available_chapters();
    		completion.actor.add(new St.Label({text: label}));
    		this.menu.addMenuItem(completion);
        }

		// TODO: finish this SWITCH option - need to investigate on saving a session with Soup
        // if (!language[Constants.LANGUAGE_CURRENT_LANGUAGE]) {
        //     let menu_switch_to = new PopupMenu.PopupBaseMenuItem();
    	// 	menu_switch_to.actor.add(new St.Label({
        //         text: _('Switch to'),
        //         x_expand: true,
        //         x_align: Clutter.ActorAlign.CENTER}));
		// 	// let switch_to_icon = new St.Icon({
    	// 	// 	icon_name: 'go-top-symbolic',
    	// 	// 	style_class: 'system-actions-icon',
    	// 	// 	icon_size: icon_size,
        //     //     y_align: Clutter.ActorAlign.CENTER
    	// 	// });
		// 	// let hbox = new St.BoxLayout();
        // 	// hbox.add_child(switch_to_icon);
    	// 	// menu_switch_to.actor.add(switch_to_icon);

    	// 	this.menu.addMenuItem(menu_switch_to);
        //     this.menu.connect('activate', Lang.bind(this, function() {
        //         duolingo.post_switch_language(this.language_code);
    	// 	}));
        // }
	},
});
